var Promise = require('promise');
var configjs = require('./config');
module.exports = function (RED) {

    /**
        hsl/rgb functions reference:
        https://stackoverflow.com/questions/2353211/hsl-to-rgb-color-conversion
     */

    /**
     * Converts an HSL color value to RGB. Conversion formula
     * adapted from http://en.wikipedia.org/wiki/HSL_color_space.
     * Assumes h, s, and l are contained in the set [0, 100] and
     * returns r, g, and b in the set [0, 255].
     *
     * @param   {number}  h       The hue
     * @param   {number}  s       The saturation
     * @param   {number}  l       The lightness
     * @return  {Array}           The RGB representation
     */
    function hslToRgb(h, s, l) {
        h /= 100; s /= 100; l /= 100;
        let r, g, b;

        if (s == 0) {
            r = g = b = l;
        } else {
            const hue2rgb = (p, q, t) => {
                if (t < 0) t += 1;
                if (t > 1) t -= 1;
                if (t < 1 / 6) return p + (q - p) * 6 * t;
                if (t < 1 / 2) return q;
                if (t < 2 / 3) return p + (q - p) * (2 / 3 - t) * 6;
                return p;
            };

            const q = l < 0.5 ? l * (1 + s) : l + s - l * s;
            const p = 2 * l - q;
            r = hue2rgb(p, q, h + 1 / 3);
            g = hue2rgb(p, q, h);
            b = hue2rgb(p, q, h - 1 / 3);
        }

        return [Math.round(r * 255), Math.round(g * 255), Math.round(b * 255)];
    }

    /**
     * Converts an RGB color value to HSL. Conversion formula
     * adapted from http://en.wikipedia.org/wiki/HSL_color_space.
     * Assumes r, g, and b are contained in the set [0, 255] and
     * returns h, s, and l in the set [0, 100].
     *
     * @param   {number}  r       The red color value
     * @param   {number}  g       The green color value
     * @param   {number}  b       The blue color value
     * @return  {Array}           The HSL representation
     */
    function rgbToHsl(r, g, b) {
        r /= 255, g /= 255, b /= 255;
        const max = Math.max(r, g, b), min = Math.min(r, g, b);
        let h, s, l = (max + min) / 2;

        if (max == min) {
            h = s = 0;
        } else {
            const d = max - min;
            s = l > 0.5 ? d / (2 - max - min) : d / (max + min);
            switch (max) {
                case r: h = (g - b) / d + (g < b ? 6 : 0); break;
                case g: h = (b - r) / d + 2; break;
                case b: h = (r - g) / d + 4; break;
            }
            h /= 6;
        }

        return [Math.round(h * 100), Math.round(s * 100), Math.round(l * 100)];
    }

    function hexToHSL(H) {
        // Convert hex to RGB first
        let r = 0, g = 0, b = 0;
        if (H.length == 4) {
            r = "0x" + H[1] + H[1];
            g = "0x" + H[2] + H[2];
            b = "0x" + H[3] + H[3];
        } else if (H.length == 7) {
            r = "0x" + H[1] + H[2];
            g = "0x" + H[3] + H[4];
            b = "0x" + H[5] + H[6];
        }
        // Then to HSL
        r /= 255;
        g /= 255;
        b /= 255;
        let cmin = Math.min(r, g, b),
            cmax = Math.max(r, g, b),
            delta = cmax - cmin,
            h = 0,
            s = 0,
            l = 0;

        if (delta == 0)
            h = 0;
        else if (cmax == r)
            h = ((g - b) / delta) % 6;
        else if (cmax == g)
            h = (b - r) / delta + 2;
        else
            h = (r - g) / delta + 4;

        h = Math.round(h * 60);

        if (h < 0)
            h += 360;

        l = (cmax + cmin) / 2;
        s = delta == 0 ? 0 : delta / (1 - Math.abs(2 * l - 1));
        s = +(s * 100).toFixed(0);
        l = +(l * 100).toFixed(0);

        return h + "," + s + "," + l;
    }

    function hsbParser(hsb) {
        var hsbArray = hsb.split(',');
        var ret = {
            h: hsbArray[0],
            s: hsbArray[1],
            b: hsbArray[2],
        }
        return ret;
    }

    function ArnidosmarthomeColorNode(config) {
        RED.nodes.createNode(this, config);

        console.debug("ArnidosmarthomeColorNode")
        console.debug(config);

        this.conf = configjs.getNode();
        this.name = config.name;
        this.device = config.device;
        this.channel = config.channel;
        // this.sethue = config.sethue;
        // this.setsaturation = config.setsaturation;
        // this.setbrightness = config.setbrightness;


        this.state = {
            value: 0,
            level: 0,
            levelUnit: "",
            color: [0, 0, 0],
            hue: 0,
            saturation: 0,
            temperatureUnit: "",
            unsingColor: true
        };

        this.reportState = function (original) {
            let msg = [{
                topic: "device",
                payload: {
                    deviceId: this.channel,
                    deviceType: "hue",
                    name: this.name,
                    value: this.state.hue
                }
            }, {
                topic: "device",
                payload: {
                    deviceId: this.channel,
                    deviceType: "saturation",
                    name: this.name,
                    value: this.state.saturation
                }
            }, {
                topic: "device",
                payload: {
                    deviceId: this.channel,
                    deviceType: "brightness",
                    name: this.name,
                    value: this.state.level
                }
            }];

            if (original !== undefined) {
                Object.assign(msg, original);
            }

            this.send(msg);
        };

        this.setState = function (value) {
            Object.assign(this.state, value);
            this.state.hue = value.h;
            this.state.saturation = value.s;
            this.state.level = value.b;
            this.state.color = [this.state.hue, this.state.saturation, this.state.level];

            this.reportState();
        };

        if (this.conf && this.device && this.channel) {
            const callback = (evt) => {
                console.debug("ColorDevice(" + this.name + ") Callback called");
                console.debug(evt);

                let state = hsbParser(evt["state"]);

                // switch (state) {
                //     case "switch":
                //         state.value = (evt["value"].toLowerCase() === "on" ? 1 : 0);
                //         break;

                //     case "level":
                //         state.level = evt["value"];
                //         break;

                //     case "saturation":
                //         state.saturation = evt["value"];
                //         state.unsingColor = true;
                //         break;

                //     case "hue":
                //         state.hue = evt["value"];
                //         state.unsingColor = true;
                //         break;

                //     case "temperature":
                //         state.temperature = evt["value"];
                //         state.unsingColor = false;
                //         break;
                // }

                this.setState(state);
            }

            this.conf.registerCallback(this, this.channel, callback);

            this.conf.getDeviceStatus(this.channel, "main").then((status) => {
                console.debug("ColorDevice(" + this.name + ") Status Refreshed");

                let state = hsbParser(status.lastState);

                this.setState(state);
            }).catch(err => {
                console.error("Ops... error getting device state (ColorDevice)");
                console.error(err);
            });

            this.on('input', msg => {
                console.debug("Input Message Received");
                console.log(msg);
                // if (this.sethue != "" || this.setsaturation != "" || this.setbrightness != "") {
                //     let com = this.sethue + "," + this.setsaturation + "," + this.setbrightness;
                //     this.conf.executeDeviceCommand({
                //         channelUid: this.channel,
                //         commandType: "HSB",
                //         command: this.sethue + "," + this.setsaturation + "," + this.setbrightness
                //     }).then((ret) => {
                //         this.setState(com);
                //     }).catch((ret) => {
                //         console.error("Error updating device");
                //     });
                // }
                if (msg && msg.color !== undefined) {
                    var color = hexToHSL(msg.color);
                    this.conf.executeDeviceCommand({
                        channelUid: this.channel,
                        commandType: "HSB",
                        command: color
                    }).then((ret) => {
                        this.setState(msg.payload.value);
                    }).catch((ret) => {
                        console.error("Error updating device");
                    });
                }
            });

            this.on('close', () => {
                console.debug("Closed");
                this.conf.unregisterCallback(this, this.channel, callback);
            });
        }
    }

    RED.nodes.registerType("arnidosmarthome-node-color", ArnidosmarthomeColorNode);

};
